/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem E. Equal Numbers                                       */
/*                                                                */
/* Original idea         Vitaly Aksenov                           */
/* Problem statement     Vitaly Aksenov                           */
/* Test set              Vitaly Aksenov                           */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Vitaly Aksenov                           */
/******************************************************************/

import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * Created by Aksenov239 on 01.10.2017.
 */
public class Tests {
    public static void main(String[] args) {
        new Tests().gen();
    }

    int MAX_N = 300_000;
    int MAX_X = 1_000_000;

    Random rnd = new Random(239);

    int tests = 1;

    public void writeTest(Value[] v) {
        try (PrintWriter out = new PrintWriter(String.format("tests/%02d", tests++))) {

            System.err.println("Writing test " + (tests - 1));

            ArrayList<Integer> a = new ArrayList<>();
            for (Value value : v) {
                for (int i = 0; i < value.cnt; i++) {
                    a.add(value.x);
                }
            }

            Collections.shuffle(a, rnd);

            out.println(a.size());
            for (int i = 0; i < a.size(); i++) {
                out.print((i == 0 ? "" : " ") + a.get(i));
            }
            out.println();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public class Value {
        int x;
        int cnt;

        public Value(int x) {
            this.x = x;
            this.cnt = 1;
        }

        public Value(int x, int cnt) {
            this.x = x;
            this.cnt = cnt;
        }
    }

    public void genSamples() {
        writeTest(new Value[] { new Value(1, 2), new Value(2, 2), new Value(3, 1), new Value(4, 1) });
    }

    public void genSmall() {
        writeTest(new Value[] {new Value(1, 1)});
        writeTest(new Value[] {new Value(MAX_X, 1)});
    }

    public int[] split(int n, int k) {
        int[] split = new int[k - 1];
        for (int i = 0; i < split.length; i++) {
            split[i] = rnd.nextInt(n - k + 1);
        }
        Arrays.sort(split);
        int[] ans = new int[k + 1];
        ans[k] = n;
        for (int i = 0; i < split.length; i++) {
            ans[i + 1] = split[i] + (i + 1);
        }
        return ans;
    }

    public Value[] merge(Value[] a, Value[] b) {
        Value[] ans = new Value[a.length + b.length];
        for (int i = 0; i < a.length; i++) {
            ans[i] = a[i];
        }
        for (int i = 0; i < b.length; i++) {
            ans[i + a.length] = b[i];
        }
        return ans;
    }

    public void writeMostCovered(int n, Value[][] all) {
        int[] split = split(n - all[1].length, all[0].length);
        for (int i = 0; i < all[0].length; i++) {
            all[0][i].cnt = split[i + 1] - split[i];
        }
        for (int i = 0; i < all[1].length; i++) {
            all[1][i].cnt = 1;
        }
        writeTest(merge(all[0], all[1]));
    }

    public void writeMostUncovered(int n, Value[][] all) {
        int[] split = split(n - all[0].length, all[1].length);
        for (int i = 0; i < all[1].length; i++) {
            all[1][i].cnt = split[i + 1] - split[i];
        }
        for (int i = 0; i < all[0].length; i++) {
            all[0][i].cnt = 1;
        }
        writeTest(merge(all[0], all[1]));
    }

    public void writeUniform(int n, Value[][] all) {
        Value[] res = merge(all[0], all[1]);
        int[] split = split(n, res.length);
        for (int i = 0; i < res.length; i++) {
            res[i].cnt = split[i + 1] - split[i];
        }
        writeTest(res);
    }

    public void genEqual(int n, int value) {
        writeTest(new Value[] { new Value(value, n) });
    }

    public Value[][] cover(Value[] v) {
        boolean[] has = new boolean[MAX_X + 1];

        for (Value value : v) {
            has[value.x] = true;
        }

        ArrayList<Value> covered = new ArrayList<>();
        ArrayList<Value> uncovered = new ArrayList<>();

        for (Value value : v) {
            int x = 2 * value.x;
            boolean isCovered = false;
            while (x <= MAX_X) {
                if (has[x]) {
                    covered.add(value);
                    isCovered = true;
                    break;
                }
                x += value.x;
            }
            if (!isCovered) {
                uncovered.add(value);
            }
        }

        return new Value[][] {
                covered.toArray(new Value[covered.size()]),
                uncovered.toArray(new Value[uncovered.size()])
        };
    }

    public Value[][] genMinAndMax(int n, int m) {
        Value[] values = new Value[n + m];
        for (int i = 0; i < n; i++) {
            values[i] = new Value(i + 1);
        }
        for (int j = 0; j < m; j++) {
            values[n + j] = new Value(MAX_X - j);
        }
        return cover(values);
    }

    public Value[][] genRandom(int n) {
        HashSet<Integer> has = new HashSet<>();
        Value[] values = new Value[n];
        for (int i = 0; i < n; i++) {
            int x = rnd.nextInt(MAX_X) + 1;
            while (has.contains(x)) {
                x = rnd.nextInt(MAX_X) + 1;
            }
            has.add(x);
            values[i] = new Value(x);
        }
        return cover(values);
    }

    public void writeAllPossible(Value[][] values) {
        writeMostCovered(MAX_N, values);
        writeMostUncovered(MAX_N, values);
        writeUniform(MAX_N, values);
    }

    public class Divisors implements Comparable<Divisors> {
        int x;
        int d;

        public Divisors(int x, int d) {
            this.x = x;
            this.d = d;
        }

        public int compareTo(Divisors d) {
            return d.d - this.d;
        }
    }

    Divisors[] divisors;

    public void precalc() {
        int[] cnt = new int[MAX_X + 1];
        Arrays.fill(cnt, 1);
        for (int i = 1; i < cnt.length; i++) {
            int x = 2 * i;
            while (x < cnt.length) {
                cnt[x]++;
                x += i;
            }
        }
        divisors = new Divisors[MAX_X];
        for (int i = 1; i < cnt.length; i++) {
            divisors[i - 1] = new Divisors(i, cnt[i]);
        }
        Arrays.sort(divisors);
    }

    public Value[][] genAllDivisors(int k) {
        HashSet<Integer> has = new HashSet<>();

        ArrayList<Value> values = new ArrayList<>();
        for (int i = 0; i < k && values.size() < MAX_N; i++) {
            if (has.contains(divisors[i].x)) {
                continue;
            }
            int x = divisors[i].x;
            values.add(new Value(x));
            for (int j = 1; j * j <= x && values.size() < MAX_N; j++) {
                if (j * j > x) {
                    break;
                }
                if (x % j != 0) {
                    continue;
                }
                if (!has.contains(j)) {
                    has.add(j);
                    values.add(new Value(j));
                }
                if (!has.contains(x / j) && values.size() < MAX_N) {
                    has.add(x / j);
                    values.add(new Value(x / j));
                }
            }
        }
        return cover(values.toArray(new Value[values.size()]));
    }

    public Value[][] genPrimes(int n) {
        boolean[] composite = new boolean[n + 1];
        ArrayList<Value> primes = new ArrayList<>();
        for (int i = 2; i * i < n && primes.size() < MAX_N; i++) {
            if (composite[i]) {
                continue;
            }
            primes.add(new Value(i));
            for (int j = i * i; j < n; j += i) {
                composite[j] = true;
            }
        }

        return cover(primes.toArray(new Value[primes.size()]));
    }

    public void gen() {
        genSamples();

        genSmall();

        genEqual(MAX_N, 1);
        genEqual(MAX_N, 2);
        genEqual(MAX_N, MAX_X);

        writeAllPossible(genMinAndMax(1, 1));
        writeUniform(MAX_N, genMinAndMax(0, MAX_N));
        writeUniform(MAX_N, genMinAndMax(MAX_N / 2, MAX_N / 2));
        writeUniform(MAX_N, genMinAndMax(MAX_N / 10, MAX_N - MAX_N / 10));
        writeUniform(MAX_N, genMinAndMax(MAX_N - MAX_N / 10, MAX_N / 10));

        for (int i = 0; i < 3; i++) {
            int l = rnd.nextInt(MAX_N / 2);
            int r = rnd.nextInt(MAX_N / 2 - l);
            writeAllPossible(genMinAndMax(l, r));
        }

        writeUniform(MAX_N, genRandom(MAX_N));
        for (int i = 0; i < 3; i++) {
            writeAllPossible(genRandom(rnd.nextInt(MAX_N / 2) + 1));
        }

        precalc();
        writeAllPossible(genAllDivisors(1));
        writeAllPossible(genAllDivisors(10));
        writeAllPossible(genAllDivisors(1000));
        writeAllPossible(genAllDivisors(10000));

        writeUniform(MAX_N, genPrimes(MAX_X));
    }
}

