/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem G. Grand Test                                          */
/*                                                                */
/* Original idea         Andrew Stankevich                        */
/* Problem statement     Borys Minaiev                            */
/* Test set              Borys Minaiev                            */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Borys Minaiev                            */
/******************************************************************/

import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.*;

public class Tests {
    private static int nTests = 0;
    private static final Path root = Paths.get("tests");
    private static final Random rnd = new Random(3242436326L);

    private static final int MAX_TESTS = 100_000;
    private static final int MAX_N = 100_000;

    public static void main(String[] args) throws IOException {
        Files.createDirectories(root);
        generateSample();
        for (int i = 0; i < 2; i++) {
            generateRandomTests(10, 10000);
            generateRandomTests(10, MAX_N);
            generateRandomTests(100, MAX_N);
            generateRandomTests(100, MAX_N);
            generateRandomTests(5, MAX_N);
        }
        for (int i = 0; i < 5; i++) {
            generateTreesWithEdges(10, MAX_N);
        }
        generateTreesWithEdges(100, MAX_N);
        for (int i = 0; i < 5; i++) {
            generateTreesWithEdges(MAX_N, MAX_N);
        }
        for (int i = 0; i < 3; i++) {
            generateTreesNoAnswer(10, MAX_N);
        }
        for (int i = 0; i < 3; i++) {
            generateTreesNoAnswer(MAX_N, MAX_N);
        }
        for (int i = 0; i < 3; i++) {
            generateTreesNoAnswerPlusEdge(100, MAX_N);
            generateTreesNoAnswerPlusEdge(1000, MAX_N);
            generateTreesNoAnswerPlusEdge(MAX_N / 2, MAX_N);
        }
        for (int size : new int[]{100, 1000, MAX_N / 2, MAX_N}) {
            generateLine(size, MAX_N, false);
            generateLine(size, MAX_N, true);
        }
        generateSpecialComps(MAX_N);
        generateSpecial3Lines(MAX_N);
    }

    private static void generateSpecialComps(int maxN) {
        ArrayList<TestCase> tc = new ArrayList<>();
        int n = (int) (0.6 * maxN);
        if (n % 2 == 0) {
            n++;
        }
        TestCase t = new TestCase(n);
        for (int i = 1; i < n; i++) {
            t.addEdge(0, i);
        }
        for (int i = 1; i < n; i += 2) {
            t.addEdge(i, i + 1);
        }
        int x = rnd.nextInt(n);
        int y = rnd.nextInt(n);
        t.addEdge(x, y);
        tc.add(t);
        test(tc);
    }

    private static void generateSpecial3Lines(int maxN) {
        ArrayList<TestCase> tc = new ArrayList<>();
        int n = maxN - 1;

        TestCase t = new TestCase(n);
        for (int i = 1; i < n; i++) {
            t.addEdge(Math.max(i - 3, 0), i);
        }
        t.addEdge(n - 1, n - 3);
        t.addEdge(n - 1, n - 2);
        tc.add(t);
        test(tc);
    }

    private static void generateLine(int maxN, int sumN, boolean biggest) {
        ArrayList<TestCase> testCases = new ArrayList<>();
        int totalN = 0, totalM = 0;
        while (testCases.size() < MAX_TESTS) {
            int n = 2 + rnd.nextInt(maxN - 2);

            TestCase t = new TestCase(n);
            for (int i = 0; i + 1 < n; i++) {
                t.addEdge(i, i + 1);
            }
            for (int x = 0; x + 1 < n; ) {
                int y = x + (biggest ? 2 : (2 + rnd.nextInt(5)));
                y = Math.min(y, n - 1);
                t.addEdge(x, y);
                x = y;
            }
            if (rnd.nextBoolean()) {
                t.addEdge(rnd.nextInt(n), rnd.nextInt(n));
            }
            totalN += t.g.length;
            totalM += t.edges.size();
            if (totalN > sumN || totalM > sumN) {
                break;
            }
            testCases.add(t);
        }
        test(testCases);
    }

    private static ArrayList<TestCase> generateTreesTestCases(int maxN, int sumN) {
        ArrayList<TestCase> testCases = new ArrayList<>();
        int totalN = 0, totalM = 0;
        while (testCases.size() < MAX_TESTS) {
            int n = 2 + rnd.nextInt(maxN - 2);
            int m = Math.min(totalM, rnd.nextInt(n) + n - 1);
            TestCase t = generateTreeTC(n, n - 1);
            t.calcParent();
            for (int i = n - 1; i < m; i++) {
                int x = rnd.nextInt(n);
                int y = x;
                int maxCnt = rnd.nextInt(n);
                while (t.parent[y] != y && !t.colored[y] && maxCnt-- > 0) {
                    t.colored[y] = true;
                    y = t.parent[y];
                }
                if (x != y) {
                    t.addEdge(x, y);
                }
            }
            totalN += t.g.length;
            totalM += t.edges.size();
            if (totalN > sumN || totalM + testCases.size() > sumN) {
                break;
            }
            testCases.add(t);
        }
        return testCases;
    }

    private static void generateTreesNoAnswer(int maxN, int sumN) {
        test(generateTreesTestCases(maxN, sumN));
    }

    private static void generateTreesNoAnswerPlusEdge(int maxN, int sumN) {
        ArrayList<TestCase> testCases = generateTreesTestCases(maxN, sumN);
        int sumE = 0;
        for (TestCase t : testCases) {
            sumE += t.edges.size();
        }
        for (TestCase t : testCases) {
            if (sumE < sumN) {
                sumE++;
                int x = rnd.nextInt(t.g.length);
                int y = rnd.nextInt(t.g.length);
                t.addEdge(x, y);
            }
        }
        test(testCases);
    }

    private static void generateSample() {
        ArrayList<TestCase> testCases = new ArrayList<>();
        testCases.add(generateSample1());
        testCases.add(generateSample2());
        test(testCases);
    }

    private static TestCase generateSample1() {
        TestCase t = new TestCase(6);
        t.addEdge(0, 1);
        t.addEdge(1, 2);
        t.addEdge(0, 2);
        t.addEdge(0, 3);
        t.addEdge(3, 2);
        t.addEdge(5, 2);
        return t;
    }

    private static TestCase generateSample2() {
        TestCase t = new TestCase(3);
        t.addEdge(0, 1);
        return t;
    }

    private static void generateRandomTests(int maxN, int sumN) {
        ArrayList<TestCase> testCases = new ArrayList<>();
        int totalN = 0, totalM = 0;
        while (testCases.size() < MAX_TESTS) {
            int n = 2 + rnd.nextInt(maxN - 2);
            int m = (int) Math.min(sumN, rnd.nextBoolean() ? (n * 2) : (n * (long) (n - 1) / 2));
            m = 1 + rnd.nextInt(m);
            TestCase t = generateRandomTest(n, m);
            totalN += t.g.length;
            totalM += t.edges.size();
            if (totalN > sumN || totalM > sumN) {
                break;
            }
            testCases.add(t);
        }
        test(testCases);
    }

    private static class DSU {
        int[] p;

        int get(int x) {
            return p[x] == x ? x : (p[x] = get(p[x]));
        }

        void unite(int x, int y) {
            p[get(x)] = get(y);
        }

        DSU(int n) {
            p = new int[n];
            for (int i = 0; i < n; i++) {
                p[i] = i;
            }
        }
    }

    private static void generateTreesWithEdges(int maxN, int sumN) {
        ArrayList<TestCase> testCases = new ArrayList<>();
        int totalN = 0, totalM = 0;
        while (testCases.size() < MAX_TESTS) {
            int n = 2 + rnd.nextInt(maxN - 2);
            int m = Math.min(totalM, rnd.nextInt(n) + n - 1);
            TestCase t = generateTreeTC(n, m);
            totalN += t.g.length;
            totalM += t.edges.size();
            if (totalN > sumN || totalM > sumN) {
                break;
            }
            testCases.add(t);
        }
        test(testCases);
    }

    private static TestCase generateTreeTC(int n, int m) {
        TestCase t = new TestCase(n);
        DSU dsu = new DSU(n);
        for (int i = 0; i + 1 < n; i++) {
            int u = rnd.nextInt(n);
            int v = rnd.nextInt(n);
            if (dsu.get(v) == dsu.get(u)) {
                i--;
                continue;
            }
            dsu.unite(v, u);
            t.addEdge(u, v);
        }
        for (int i = 0; i < m - (n - 1); i++) {
            int u = rnd.nextInt(n);
            int v = rnd.nextInt(n);
            t.addEdge(u, v);
        }
        return t;
    }

    private static TestCase generateRandomTest(int n, int m) {
        TestCase t = new TestCase(n);
        for (int i = 0; i < m; i++) {
            int x = rnd.nextInt(n);
            int y = rnd.nextInt(n);
            if (x == y) {
                i--;
                continue;
            }
            t.addEdge(x, y);
        }
        return t;
    }

    private static class TestCase {
        List<Integer>[] g;
        boolean[] colored;

        TestCase(int n) {
            g = new ArrayList[n];
            for (int i = 0; i < n; i++) {
                g[i] = new ArrayList<>();
            }
        }

        int[] parent;

        void calcParent() {
            colored = new boolean[g.length];
            parent = new int[g.length];
            Arrays.fill(parent, -1);
            for (int i = 0; i < g.length; i++) {
                if (parent[i] != -1) {
                    continue;
                }
                calcParent(i, i);
            }
        }

        void calcParent(int v, int p) {
            parent[v] = p;
            for (int to : g[v]) {
                if (parent[to] == -1) {
                    calcParent(to, v);
                }
            }
        }

        HashSet<Long> edges = new HashSet<>();

        private boolean addEdge(int fr, int to) {
            if (fr == to) {
                return false;
            }
            if (fr > to) {
                int tmp = fr;
                fr = to;
                to = tmp;
            }
            long hash = ((long) fr << 30L) + to;
            if (edges.contains(hash)) {
                return false;
            }
            g[fr].add(to);
            g[to].add(fr);
            edges.add(hash);
            return true;
        }
    }

    private static int[] genPermutation(int n) {
        int[] a = new int[n];
        for (int i = 0; i < a.length; i++) {
            int j = rnd.nextInt(i + 1);
            a[i] = a[j];
            a[j] = i;
        }
        return a;
    }

    private static void test(List<TestCase> tests) {
        try (PrintWriter out = new PrintWriter(String.format("%s/%02d", root, ++nTests))) {
            System.out.println("Writing test " + nTests);
            out.println(tests.size());
            for (TestCase t : tests) {
                int m = t.edges.size();
                out.println(t.g.length + " " + m);
                int n = t.g.length;
                int[] perm = genPermutation(n);
                for (int x : perm) {
                    int[] perm2 = genPermutation(t.g[x].size());
                    for (int id2 : perm2) {
                        int y = t.g[x].get(id2);
                        if (y > x) {
                            out.println((x + 1) + " " + (y + 1));
                        }
                    }
                }
            }
        } catch (IOException e) {
            throw new AssertionError(e);
        }
    }
}

