/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem H. Hidden Supervisors                                  */
/*                                                                */
/* Original idea         Niyaz Nigmatullin                        */
/* Problem statement     Niyaz Nigmatullin                        */
/* Test set              Niyaz Nigmatullin                        */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Niyaz Nigmatullin                        */
/******************************************************************/

import java.io.PrintStream
import java.util.*

private var tests = 0
private val rand = Random(12345678)

fun main(args: Array<String>) {
    if (args.size == 3 && args[0] == "stdout") {
        rand.setSeed(Arrays.hashCode(args).toLong())
        for (i in 0..100) {
            genRandom(args[1].toInt()).printStdout()
        }
        return
    }
    val MAXN = 100000
    // Sample testcases
    Test(intArrayOf(-1, 2, 0, -1, 3, 3)).print()
    Test(intArrayOf(-1, 2, 0, -1, 5, 3)).print()
    //
    genSpecial3(100, 10, 6).print()
    genSpecial3(1000, 300, 60).print()
    genSpecial3(10000, 128, 10).print()
    genSpecial3(20000, 10, 66).print()
    genSpecial3(30000, 654, 32).print()
    genSpecial3(40000, 154, 32).print()
    genSpecial3(50000, 154, 42).print()
    genSpecial3(75000, 8888, 777).print()
    genSpecial3(100000, 2328, 1233).print()
    genSpecial3(75000, 8888, 111).print()
    genSpecial3(100000, 2328, 123).print()
    for (i in listOf(11, 50, 1024, MAXN)) {
        genEmpty(i).print()
        genRandom(i).print()
        genRandom2(i).shuffled().print()
        genSpecial2(i).print()
        genSpecial1(i).print()
        genSpecial0(i).print()
        genPerfectRoot(i).print()
    }
    genTestTrees(genBamboo2(MAXN)).print()
    genTestTrees(mergeTrees(genTreePerfect(4), genTreePerfectFreeRoot(11), genBamboo2(MAXN - 123))).print()
    genTestTrees(genBamboo(MAXN)).print()
    genTestTrees(genBamboo(MAXN - 1)).print()
    for (i in listOf(16, 330, 512, 1777, 12347, 23487, 74320, 77777, 98765, MAXN - 1)) {
        genSpecial2(i).print()
        genSpecial1(i).print()
    }
    for (i in listOf(MAXN, 1000, 10, 4, 2)) {
        genBunchOfPerfect(i and 1.inv()).print()
    }
}

private fun genPermutationNoZero(n: Int): IntArray {
    assert(n >= 0)
    val ret = IntArray(n)
    for (i in 1 until n) {
        val j = rand.nextInt(i) + 1
        ret[i] = ret[j]
        ret[j] = i
    }
    assert(ret[0] == 0 && ret.toSet().size == n && ret.all { it in 0 until n })
    return ret
}

private class Test(private val parent: IntArray) {
    fun print() = PrintStream("tests/%02d".format(++tests)).use { printTo(it) }

    fun printStdout() = printTo(System.out)

    private fun printTo(out: PrintStream) = with(out) {
        println(parent.size)
        println(parent.drop(1).joinToString(" ", transform = { (it + 1).toString() }))
        System.err.print("[$tests]")
    }

    fun shuffled(): Test {
        val n = parent.size
        val r = genPermutationNoZero(n)
        val p = IntArray(n)
        for (i in 0 until n) {
            p[r[i]] = when {
                parent[i] >= 0 -> r[parent[i]]
                else -> -1
            }
        }
        return Test(p)
    }
}

private fun genEmpty(n: Int) = Test(IntArray(n) { -1 })

private fun genRandom(n: Int) = Test(IntArray(n) { rand.nextInt(it + 1) - 1 })

private fun genRandom2(n: Int) = Test(IntArray(n) { rand.nextInt(rand.nextInt(it + 1) + 1) - 1 })

private class Counter(var current: Int = 0) {
    fun get() = current++
}

private class Tree {
    val children = arrayListOf<Tree>()
    var id = -1

    fun addChild(t: Tree) {
        children.add(t)
    }

    fun sign(c: Counter) {
        id = c.get()
        children.forEach { it.sign(c) }
    }

    fun printToArray(p: IntArray) {
        for (e in children) {
            e.printToArray(p)
            p[e.id] = id
        }
    }

    fun printToList(list: ArrayList<Tree>) {
        list.add(this)
        children.forEach { it.printToList(list) }
    }

    fun isFree(): Boolean = !children.any { it.isFree() }

    fun maxMatching(): Pair<Int, Boolean> = children.map { it.maxMatching() }.fold(Pair(0, true), { a, b ->
        val value = a.first + b.first + (if (a.second && b.second) 1 else 0)
        val free = a.second && !b.second
        Pair(value, free)
    })

    fun size(): Int = 1 + children.sumBy { it.size() }

    override fun toString(): String = "(" + children.map { it.toString() } + ")"
}

private fun makeFreeRoot(t: Tree) = when {
    t.isFree() -> t
    else -> mergeTrees(Tree(), t)
}

private fun makeNonFreeRoot(t: Tree) = when {
    t.isFree() -> mergeTrees(Tree(), t)
    else -> t
}

private fun genBamboo(n: Int): Tree {
    val root = Tree()
    var v = root
    for (i in 2..n) {
        val u = Tree()
        v.children.add(u)
        v = u
    }
    assert(root.size() == n)
    return root
}

private fun genBamboo2(n: Int): Tree {
    val root = Tree()
    var v = root
    for (i in 2..n / 2) {
        val u = Tree()
        v.children.add(u)
        if (i % 2 == 0)
            v.children.add(Tree())
        v = u
    }
    assert(root.size() <= n)
    return root
}

private fun genTreePerfectFreeRoot(n: Int): Tree {
    assert(n % 2 == 1, { "perfect except root exists only if n % 2 == 1" })
    var left = n - 1
    val root = Tree()
    while (left > 0) {
        val size = 2 * (rand.nextInt(left / 2) + 1)
        root.addChild(genTreePerfect(size))
        left -= size
    }
    assert(root.size() == n && root.maxMatching() == Pair(n / 2, true),
            { "n = $n, size = ${root.size()}, " + root.maxMatching().toString() + ", $root" })
    return root
}

private fun genTreePerfect(n: Int): Tree {
    assert(n > 0 && n % 2 == 0, { "perfect matching exists only if n > 0 and n % 2 == 0" })
    val first = 2 * rand.nextInt(n / 2) + 1
    val root = genTreePerfectFreeRoot(first)
    root.addChild(genTreePerfectFreeRoot(n - first))
    assert(root.size() == n && root.maxMatching().first == n / 2,
            { "n = $n, size = ${root.size()}, " + root.maxMatching().toString() })
    return root
}

private fun genTestTrees(vararg trees: Tree): Test {
    assert(trees.isNotEmpty())
    val c = Counter()
    trees.forEach { it.sign(c) }
    val p = IntArray(c.get()) { -1 }
    assert(p.size == trees.sumBy(Tree::size))
    trees.forEach { it.printToArray(p) }
    return Test(p).shuffled()
}

private fun genBunchOfPerfect(n: Int): Test {
    assert(n % 2 == 0)
    val trees = arrayListOf<Tree>()
    var left = n
    while (left > 0) {
        val size = 2 * (rand.nextInt(left / 2) + 1)
        trees.add(genTreePerfect(size))
        left -= size
    }
    Collections.shuffle(trees, rand)
    assert(n == trees.sumBy(Tree::size))
    assert(n / 2 == trees.sumBy { it.maxMatching().first })
    return genTestTrees(*trees.toTypedArray())
}


private fun mergeTrees(vararg trees: Tree): Tree {
    assert(trees.isNotEmpty())
    val sumSizes = trees.sumBy { it.size() }
    val list = arrayListOf<Tree>()
    for (t in trees) {
        if (list.isNotEmpty()) {
            list[rand.nextInt(list.size)].addChild(t)
        }
        t.printToList(list)
    }
    assert(trees[0].size() == sumSizes)
    return trees[0]
}

private fun getRandomTree(n: Int) = mergeTrees(*Array(n) { Tree() })

private fun genPerfectRoot(n: Int): Test {
    val rootSize = 2 * (rand.nextInt(n / 3) + 1)
    val trees = arrayListOf<Tree>()
    trees.add(genTreePerfect(rootSize))
    var left = n - rootSize
    while (left > 1) {
        val size = 2 * rand.nextInt(left / 2) + 1
        trees.add(genTreePerfectFreeRoot(size))
        left -= size
    }
    if (left > 0) {
        trees.add(Tree())
    }
    return genTestTrees(*trees.toTypedArray())
}

private fun genSpecial1(n: Int): Test {
    val s = 2 * (rand.nextInt(n / 5) + 1)
    val list = arrayListOf(genTreePerfect(s))
    var left = n - s
    while (left > 20) {
        val size = rand.nextInt(Math.min(left - 1, 20)) + 1
        val tree = makeFreeRoot(getRandomTree(size))
        list.add(tree)
        left -= tree.size()
    }
    assert(list.sumBy(Tree::size) <= n)
    return genTestTrees(*list.toTypedArray())
}

private fun genSpecial2(n: Int): Test {
    val s = 2 * (rand.nextInt(n / 6) + 1)
    val list = arrayListOf(genTreePerfect(s))
    var left = n - s
    while (left > 20) {
        val size = rand.nextInt(Math.min(left - 1, 20)) + 2
        val nTree = when (rand.nextInt(3)) {
            0 -> makeFreeRoot(getRandomTree(size))
            1 -> genTreePerfect(size and 1.inv())
            else -> genTreePerfectFreeRoot(size or 1)
        }
        list.add(nTree)
        left -= nTree.size()
    }
    assert(list.sumBy(Tree::size) <= n)
    return genTestTrees(*list.toTypedArray())
}

private fun genSpecial3(n: Int, s: Int, smallSize: Int): Test {
    assert(s % 2 == 0)
    val list = arrayListOf(genTreePerfect(s))
    var left = n - s
    while (left > smallSize / 5 && left > 5) {
        val size = rand.nextInt(Math.min(left - 2, smallSize)) + 2
        val nTree = when (rand.nextInt(3)) {
            0 -> makeFreeRoot(getRandomTree(size))
            1 -> genTreePerfect(size and 1.inv())
            else -> genTreePerfectFreeRoot(size or 1)
        }
        list.add(nTree)
        left -= nTree.size()
    }
    assert(list.sumBy(Tree::size) <= n)
    return genTestTrees(*list.toTypedArray())
}

private fun genSpecial0(n: Int): Test {
    val s = 2 * (rand.nextInt(n / 5) + 1)
    return genTestTrees(genTreePerfect(s), makeFreeRoot(getRandomTree(n - s - 1)))
}

