/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem H. Hidden Supervisors                                  */
/*                                                                */
/* Original idea         Niyaz Nigmatullin                        */
/* Problem statement     Niyaz Nigmatullin                        */
/* Test set              Niyaz Nigmatullin                        */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Niyaz Nigmatullin                        */
/******************************************************************/

#include <cstdio>
#include <vector>
#include <algorithm>
#include <cassert>
#include <functional>
using namespace std;

struct matching {
  int size;
  int free;
  bool root_free;
};

vector<vector<int> > edges;

matching dfs(int v) {
  bool can = true;
  int have = 1;
  int size = 0;
  for (int to : edges[v]) {
    matching got = dfs(to);
    if (got.root_free && can) {
      can = false;
      have--;
      got.free--;
      size++;
    }
    size += got.size;
    have += got.free;
  }
  return {size, have, can};
}

bool add_free(vector<int> &free_vertices, int v) {
  bool is_free = true;
  for (int to : edges[v]) {
    if (add_free(free_vertices, to)) {
      if (is_free) {
        is_free = false;
      } else {
        free_vertices.push_back(to);
      }
    }
  }
  return is_free;
}

int main() {
  assert(freopen("hidden.in", "r", stdin));
  assert(freopen("hidden.out", "w", stdout));
  int n;
  while (scanf("%d", &n) == 1) {
    edges.clear();
    edges.resize(n);
    vector<int> roots;
    vector<int> answer(n);
    for (int i = 1; i < n; i++) {
      int x;
      assert(scanf("%d", &x) == 1);
      --x;
      if (x < 0) {
        roots.push_back(i);
      } else {
        edges[x].push_back(i);
      }
      answer[i] = x;
    }
    vector<pair<int, int> > free_roots;
    matching m0 = dfs(0);
    int ans = m0.size;
    vector<int> free_vertices;
    if (add_free(free_vertices, 0)) {
      free_vertices.push_back(0);
    }
    for (int v : roots) {
      matching got = dfs(v);
      ans += got.size;
      if (!got.root_free) {
        add_free(free_vertices, v);
        answer[v] = 0;
      } else {
        free_roots.push_back({got.free, v});
      }
    }
    std::sort(free_roots.begin(), free_roots.end(), std::greater<pair<int, int> >());
    for (pair<int, int> cur : free_roots) {
      int v = cur.second;
      if (!free_vertices.empty()) {
        ans++;
        answer[v] = free_vertices.back();
        free_vertices.pop_back();
        add_free(free_vertices, v);
      } else {
        answer[v] = 0;
        if (add_free(free_vertices, v)) {
          free_vertices.push_back(v);
        }
      }
    }
    printf("%d\n", ans);
    for (int i = 1; i < n; i++) {
      if (i > 1) putchar(' ');
      printf("%d", answer[i] + 1);
    }
    puts("");
  }
}

