/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem J. Joker                                               */
/*                                                                */
/* Original idea         Gennady Korotkevich                      */
/* Problem statement     Borys Minaiev                            */
/* Test set              Borys Minaiev                            */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Borys Minaiev                            */
/******************************************************************/

import java.io.IOException;
import java.io.PrintWriter;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.Random;

public class Tests {
    private static int nTests = 0;
    private static final Path root = Paths.get("tests");
    private static final Random rnd = new Random(354353452);
    private static final int MAX_SUM = (int) 1e9;
    private static final int MAX_N = 50_000;

    public static void main(String[] args) throws IOException {
        Files.createDirectories(root);
        test(generateSample());
        for (int i = 0; i < 5; i++) {
            test(generateRandomTestCase(100, false));
        }
        for (int i = 0; i < 3; i++) {
            test(generateRandomTestCase(MAX_N, false));
        }
        for (int i = 0; i < 3; i++) {
            test(generateRandomTestCase(MAX_N, true));
        }
        test(generateBigConvexTestCase(MAX_N, false));
        test(generateBigConvexTestCase(MAX_N, true));
        test(generateBigConvexTestCase(MAX_N, true));
        test(generateBigConvexRandomChangesTestCase(MAX_N, false));
        test(generateBigConvexRandomChangesTestCase(MAX_N, true));
        test(generateSpecial1(MAX_N));
        test(generateSpecial2(MAX_N));
        for (int hullSize : new int[]{200, 300, 400, 500, 600, 750, 1000, 1500, 2000}) {
            test(generateConvexHulls(MAX_N, hullSize, false));
            test(generateConvexHulls(MAX_N, hullSize, true));
        }
    }

    private static TestCase generateSpecial1(int n) {
        int[] a = new int[n];
        for (int i = 0; i < n; i++) {
            a[i] = i % 2 == 0 ? -1 : 1;
        }
        TestCase t = new TestCase(a, n);
        for (int i = 0; i < n; i += 2) {
            int pos = rnd.nextInt(n);
            t.pos[i] = pos;
            t.value[i] = 10;
            t.pos[i + 1] = pos;
            t.value[i + 1] = a[pos];
        }
        return t;
    }

    private static TestCase generateSpecial2(int n) {
        int[] a = new int[]{123, -123};
        TestCase t = new TestCase(a, n);
        for (int i = 0; i < n; i++) {
            t.pos[i] = 0;
            t.value[i] = rnd.nextInt(MAX_SUM) + 1;
        }
        return t;
    }

    private static TestCase generateSample() {
        TestCase t = new TestCase(new int[]{1, -5, 3, -5}, 7);
        t.pos = new int[]{3, 1, 2, 3, 0, 1, 2};
        t.value = new int[]{-1, -1, 10, 10, -1, 1, -1};
        return t;
    }

    private static int[] generatePositiveNumbers(int sum, int cnt) {
        if (cnt == 1) {
            return new int[]{sum};
        }
        sum -= cnt;
        if (sum < 0) {
            throw new AssertionError();
        }
        int[] vals = new int[cnt - 1];
        for (int i = 0; i < vals.length; i++) {
            vals[i] = rnd.nextInt(sum + 1);
        }
        Arrays.sort(vals);
        int[] res = new int[cnt];
        for (int i = 0; i + 1 < cnt; i++) {
            res[i] = (vals[i] - (i == 0 ? 0 : vals[i - 1])) + 1;
        }
        res[res.length - 1] = sum - vals[vals.length - 1] + 1;
        return res;
    }

    private static int gcd(int x, int y) {
        return x == 0 ? y : gcd(y % x, x);
    }

    private static class Point implements Comparable<Point> {
        int x, y;

        public Point(int x, int y) {
            this.x = x;
            this.y = y;
        }

        @Override
        public int compareTo(Point o) {
            long value = x * (long) o.y - y * (long) o.x;
            return -Long.signum(value);
        }
    }

    private static int[] generateConvexHull(int maxN) {
        int[] a = new int[maxN];
        ArrayList<Point> vectors = new ArrayList<>();
        int sumX = 0, sumY = 0;
        out:
        for (int sum = 1; ; sum++) {
            for (int dy = 1; dy < sum; dy++) {
                int dx = sum - dy;
                if (vectors.size() * 2 + 1 >= maxN) {
                    break out;
                }
                if (sumX * 2 > MAX_SUM || sumY * 2 > MAX_SUM) {
                    break;
                }
                if (gcd(dx, dy) == 1) {
                    sumX += dx;
                    sumY += dy;
                    vectors.add(new Point(dx, dy));
                }
            }
        }
        Collections.sort(vectors);
        for (int i = 0; i < vectors.size(); i++) {
            a[i * 2 + 1] = vectors.get(i).x;
            a[i * 2] = -vectors.get(i).y;
        }
        for (int i = vectors.size() * 2; i < a.length; i++) {
            a[i] = -1;
        }
        return a;
    }

    private static TestCase generateBigConvexTestCase(int maxN, boolean randomLast) {
        int[] a = generateConvexHull(maxN);
        int sumX = 0;
        int sumY = 0;
        for (int z : a) {
            if (z > 0) {
                sumX += z;
            } else {
                sumY -= z;
            }
        }
        TestCase t = new TestCase(a, maxN);
        for (int i = 0; i < maxN; i++) {
            t.pos[i] = a.length - 1;
            int to = (MAX_SUM - sumX) / 2;
            int from = -(MAX_SUM - sumY) / 2;
            if (randomLast) {
                t.value[i] = from + rnd.nextInt(to - from);
            } else {
                t.value[i] = i % 2 == 0 ? from : to;
            }
        }
        return t;
    }

    private static TestCase generateConvexHulls(int maxN, int hullSize, boolean changeOnlyLast) {
        int[] hull = generateConvexHull(hullSize);
        int[] a = new int[maxN];
        for (int i = 0; i < a.length; i++) {
            a[i] = hull[i % hull.length];
        }
        TestCase t = new TestCase(a, maxN);
        int sumX = 0;
        int sumY = 0;
        for (int z : a) {
            if (z > 0) {
                sumX += z;
            } else {
                sumY -= z;
            }
        }
        int diff = sumX - sumY;
        for (int i = 0; i < maxN; i++) {
            if (changeOnlyLast || rnd.nextBoolean()) {
                t.pos[i] = a.length - 1;
                int to = -1;
                int from = -diff;
                t.value[i] = from + rnd.nextInt(to - from);
            } else {
                int pos = rnd.nextInt(a.length - 1);
                int curValue = a[pos];
                int nextValue = 1 + rnd.nextInt(Math.abs(curValue));
                if (curValue > 0) {
                    sumX -= curValue;
                    sumX += nextValue;
                } else {
                    sumY += curValue;
                    sumY += nextValue;
                    nextValue *= -1;
                }
                t.pos[i] = pos;
                t.value[i] = nextValue;
                a[pos] = nextValue;
            }
        }
        return t;
    }

    private static TestCase generateBigConvexRandomChangesTestCase(int maxN, boolean randomLast) {
        int[] a = generateConvexHull(maxN);
        int sumX = 0;
        int sumY = 0;
        for (int z : a) {
            if (z > 0) {
                sumX += z;
            } else {
                sumY -= z;
            }
        }
        TestCase t = new TestCase(a, maxN);
        for (int i = 0; i < maxN; i++) {
            if (rnd.nextBoolean()) {
                int pos = rnd.nextInt(a.length - 1);
                int curValue = a[pos];
                int nextValue = 1 + rnd.nextInt(Math.abs(curValue));
                if (curValue > 0) {
                    sumX -= curValue;
                    sumX += nextValue;
                } else {
                    sumY += curValue;
                    sumY += nextValue;
                    nextValue *= -1;
                }
                t.pos[i] = pos;
                t.value[i] = nextValue;
                a[pos] = nextValue;
            } else {
                t.pos[i] = a.length - 1;
                int to = (MAX_SUM - sumX) / 2;
                int from = -(MAX_SUM - sumY) / 2;
                if (randomLast) {
                    t.value[i] = from + rnd.nextInt(to - from);
                } else {
                    t.value[i] = i % 2 == 0 ? from : to;
                }
            }
        }
        return t;
    }

    private static TestCase generateRandomTestCase(int maxN, boolean isMaxTest) {
        int n = isMaxTest ? maxN : (2 + rnd.nextInt(maxN));
        int positiveNumbers = 1 + rnd.nextInt(n - 1);
        int negativeNumers = n - positiveNumbers;
        int sumPositive = Math.max(positiveNumbers, rnd.nextInt(MAX_SUM));
        int sumNegative = Math.max(negativeNumers, rnd.nextInt(MAX_SUM));
        int[] pos = generatePositiveNumbers(sumPositive, positiveNumbers);
        int[] neg = generatePositiveNumbers(sumNegative, negativeNumers);
        int[] a = new int[n];
        int curPositive = positiveNumbers, curNegative = negativeNumers;
        for (int i = 0; i < n; i++) {
            double probPositive = curPositive / (curPositive + curNegative + 0.0);
            if (rnd.nextDouble() < probPositive) {
                a[i] = pos[--curPositive];
            } else {
                a[i] = -neg[--curNegative];
            }
        }
        int[] realValues = a.clone();
        int changes = isMaxTest ? maxN : (1 + rnd.nextInt(maxN));
        TestCase t = new TestCase(a, changes);
        sumPositive = 0;
        sumNegative = 0;
        curPositive = 0;
        curNegative = 0;
        for (int i = 0; i < n; i++) {
            if (a[i] > 0) {
                sumPositive += a[i];
                curPositive++;
            } else {
                sumNegative -= a[i];
                curNegative++;
            }
        }
        for (int i = 0; i < changes; i++) {
            if (sumPositive <= 0 || sumNegative <= 0) {
                throw new AssertionError();
            }
            t.pos[i] = rnd.nextInt(n);
            int position = t.pos[i];
            boolean makePositive = true;
            if (realValues[position] > 0) {
                if (curPositive == 1 || rnd.nextBoolean()) {
                    makePositive = true;
                } else {
                    makePositive = false;
                }
                curPositive--;
                sumPositive -= realValues[position];
            } else {
                if (curNegative == 1 || rnd.nextBoolean()) {
                    makePositive = false;
                } else {
                    makePositive = true;
                }
                curNegative--;
                sumNegative += realValues[position];
            }
            if (sumPositive == MAX_SUM && makePositive) {
                makePositive = false;
            }
            if (sumNegative == MAX_SUM && !makePositive) {
                makePositive = true;
            }
            if (makePositive) {
                curPositive++;
                int maxCanBe = MAX_SUM - sumPositive;
                t.value[i] = 1 + rnd.nextInt(maxCanBe);
                sumPositive += t.value[i];
            } else {
                curNegative++;
                int maxCanBe = MAX_SUM - sumNegative;
//                System.err.println("s = " + sumPositive + " " + sumNegative + " " + curPositive + " " + curNegative);
                t.value[i] = -1 - rnd.nextInt(maxCanBe);
                sumNegative -= t.value[i];
            }
            realValues[t.pos[i]] = t.value[i];
        }
        return t;
    }

    private static void test(final TestCase t) {
        try (PrintWriter out = new PrintWriter(String.format("%s/%02d", root, ++nTests))) {
            System.out.println("Writing test " + nTests);
            t.print(out);
        } catch (IOException e) {
            throw new AssertionError(e);
        }
    }

    private static class TestCase {
        int[] a;
        int[] pos;
        int[] value;

        TestCase(int[] a, int changes) {
            pos = new int[changes];
            value = new int[changes];
            this.a = a.clone();
        }

        void print(PrintWriter out) {
            out.println(a.length + " " + pos.length);
            for (int i = 0; i < a.length; i++) {
                out.print(a[i]);
                if (i != a.length - 1) {
                    out.print(" ");
                } else {
                    out.println();
                }
            }
            for (int i = 0; i < pos.length; i++) {
                out.println((1 + pos[i]) + " " + value[i]);
            }
        }
    }
}

