/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem J. Joker                                               */
/*                                                                */
/* Original idea         Gennady Korotkevich                      */
/* Problem statement     Borys Minaiev                            */
/* Test set              Borys Minaiev                            */
/******************************************************************/
/* Solution                                                       */
/*                                                                */
/* Author                Georgiy Korneev                          */
/******************************************************************/

import java.io.*;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

public class joker_gk {
    static Scanner in;
    static PrintWriter out;
    static int SIZE = 600;
    static long OPS = 0;

    static class Chunk {
        int i, xo, yo, c;
        int[] x = new int[SIZE];
        int[] y = new int[SIZE];
        int[] hull = new int[SIZE];

        Chunk(final int i) {
            this.i = i;
        }

        void hull() {
            c = 0;
            hull[c++] = 0;
            hull[c++] = 1;
            out: for (int i = 2; i < x.length; i++) {
                while (c >= 2) {
                    int dx = x[i] - x[hull[c - 1]];
                    int dy = y[i] - y[hull[c - 1]];
                    if (dx == 0 && dy == 0) {
                        continue out;
                    }
                    int pdx = x[hull[c - 1]] - x[hull[c - 2]];
                    int pdy = y[hull[c - 1]] - y[hull[c - 2]];
                    OPS++;
                    if (dx * (long) pdy >= dy * (long) pdx) {
                        c--;
                    } else {
                        break;
                    }
                }
                hull[c++] = i;
            }
        }

        private long eval(final int i, final long tp, final long tn) {
            OPS++;
            return (xo + x[hull[i]]) * tn - (yo + y[hull[i]]) * tp;
        }

        int best(final long tp, final long tn) {
            int l = 0;
            int r = c - 1;
            while (l < r - 1) {
                int a = (l * 2 + r) / 3;
                int b = (l + r * 2) / 3;
                if (eval(a, tp, tn) < eval(b, tp, tn)) {
                    l = a + 1;
                } else {
                    r = b - 1;
                }
            }
            return eval(l, tp, tn) >= eval(r, tp, tn) ? l : r;
        }
    }

    static class Solution {
        void run() {
            long START = System.currentTimeMillis();
            int n = in.nextInt();
            int m = in.nextInt();
            int[] a = new int[n];
            int[] p = new int[m];
            int[] v = new int[m];
            for (int i = 0; i < n; i++) {
                a[i] = in.nextInt();
            }
            for (int i = 0; i < m; i++) {
                p[i] = in.nextInt() - 1;
                v[i] = in.nextInt();
            }
            for (int i : solve(a, p, v)) {
                out.println(i + 1);
            }
            System.err.println("done in " + (System.currentTimeMillis() - START) + " ms");
        }

        int[] solve(int[] a, int[] p, int[] v) {
            int x = 0;
            int y = 0;
            List<Chunk> chunks = new ArrayList<>();
            for (int i = 0; i < a.length; ) {
                final Chunk chunk = new Chunk(i);
                for (int j = 0; j < SIZE; j++, i++) {
                    final int q = i < a.length ? a[i] : 0;
                    if (q > 0) {
                        x += q;
                    } else {
                        y -= q;
                    }
                    chunk.x[j] = x;
                    chunk.y[j] = y;
                }
                chunk.hull();
                chunks.add(chunk);
            }

            int[] answer = new int[p.length + 1];
            answer[0] = solve(chunks);
            for (int i = 0; i < p.length; i++) {
                int pd = 0;
                int nd = 0;
                if (a[p[i]] > 0) {
                    pd -= a[p[i]];
                } else {
                    nd += a[p[i]];
                }
                a[p[i]] = v[i];
                if (a[p[i]] > 0) {
                    pd += a[p[i]];
                } else {
                    nd -= a[p[i]];
                }

                for (int j = p[i] / SIZE + 1; j < chunks.size(); j++) {
                    final Chunk chunk = chunks.get(j);
                    chunk.xo += pd;
                    chunk.yo += nd;
                    OPS++;
                }
                final Chunk chunk = chunks.get(p[i] / SIZE);
                for (int j = p[i] % SIZE; j < SIZE; j++) {
                    chunk.x[j] += pd;
                    chunk.y[j] += nd;
                    OPS++;
                }
                chunk.hull();

                answer[i + 1] = solve(chunks);
            }
            return answer;
        }

        int solve(final List<Chunk> chunks) {
            final Chunk last = chunks.get(chunks.size() - 1);
            long tp = last.xo + last.x[SIZE - 1];
            long tn = last.yo + last.y[SIZE - 1];

            long best = Long.MIN_VALUE;
            int bestI = -1;
            for (Chunk chunk : chunks) {
                int i = chunk.best(tp, tn);
                long v = chunk.eval(i, tp, tn);
                if (best < v) {
                    best = v;
                    bestI = chunk.i + chunk.hull[i];
                }
                OPS++;
            }

            return bestI;
        }

    }

    public static void main(String[] args) throws Exception {
        in = new Scanner(new File("joker.in"));
        out = new PrintWriter("joker.out");

        new Solution().run();

        in.close();
        out.close();
    }

    static class Scanner {
        final BufferedReader reader;
        StringTokenizer tokenizer = new StringTokenizer("");

        Scanner(File file) throws FileNotFoundException {
            reader = new BufferedReader(new FileReader(file));
        }

        int nextInt() {
            return Integer.parseInt(next());
        }

        String next() {
            while (!tokenizer.hasMoreTokens()) {
                try {
                    tokenizer = new StringTokenizer(reader.readLine());
                } catch (IOException e) {
                    throw new AssertionError();
                }
            }
            return tokenizer.nextToken();
        }

        void close() {
        }
    }
}

