/******************************************************************/
/* ACM ICPC 2017-2018, NEERC                                      */
/* Northern Subregional Contest                                   */
/* St Petersburg, November 4, 2017                                */
/******************************************************************/
/* Problem L. Little Difference                                   */
/*                                                                */
/* Original idea         Dmitry Yakutov                           */
/* Problem statement     Pavel Kunyavskiy                         */
/* Test set              Pavel Kunyavskiy                         */
/******************************************************************/
/* Tests                                                          */
/*                                                                */
/* Author                Pavel Kunyavskiy                         */
/******************************************************************/

import java.io.IOException;
import java.io.PrintWriter;
import java.math.BigInteger;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Random;

public class Tests {
    private static final Path root = Paths.get("tests");
    private static final long MAXN = 1_000_000_000_000_000_000L;
    private static int nTests;
    private static Random rnd;
    private static int have_answer[];

    public static void main(String[] args) throws IOException {
        Files.createDirectories(root);
        test(12);
        test(1);
        test(MAXN);
        for (int i = 2; i <= 10; i++) {
            test(i);
        }
        rnd = new Random(239017);
        randomTests();
        genPow2();
        for (int i = 0; i < 5; i++) {
            int a = rnd.nextInt(1000);
            genProd(a);
        }
        for (int i = 0; i < 3; i++) {
            int a = rnd.nextInt(1_000_000_000);
            genProd(a);
        }
        genProd(2);
        int[] primes = getPrimes();
        have_answer = new int[100];
        genManyDivisors(primes, 0, 1, 100);
        test(genPrime(1_000_000_000) * genPrime(1_000_000_000));
        test(genPrime(MAXN));
    }

    private static void genManyDivisors(int[] primes, int pos, long val, int last_deg) {
        try_test(val);
        if (pos == primes.length) {
            return;
        }
        int deg;
        for (deg = 1; deg <= last_deg; deg++) {
            if (val > MAXN / primes[pos]) {
                break;
            }
            val *= primes[pos];
        }
        for (deg--; deg > 0; deg--) {
            genManyDivisors(primes, pos + 1, val, deg);
            if (val % primes[pos] != 0) {
                throw new RuntimeException();
            }
            val /= primes[pos];
        }
        if (val % primes[pos] == 0) {
            throw new RuntimeException();
        }
    }

    private static void try_test(long val) {
        int res = solve(val);
        if (res > 0 && have_answer[res] < 2) {
            System.out.println("Found test " + val + " with answer " + res);
            test(val);
            have_answer[res]++;
        }
    }

    private static int solve(long n) {
        if ((n & (n - 1)) == 0) {
            return -1;
        }

        ArrayList<Long> candidates = new ArrayList<Long>();
        candidates.add(n);
        for (int s = 1; s <= 64; s++) {
            long r = (long)Math.pow((double)n, 1.0 / s);
            for (int j = -2; j <= 2; j++) {
                long c = r + j;
                if (c > 1 && n % c == 0) {
                    candidates.add(c);
                }
            }
        }
        Collections.sort(candidates);
        long last = -1;
        int ans = 0;
        for (long x : candidates) {
            if (x == last) {
                continue;
            }
            if (solveOne(n, x)) {
                ans++;
            }
            last = x;
        }
        return ans;
    }

    private static boolean solveOne(long n, long x) {
        while (n % x == 0) n /= x;
        while (n % (x + 1) == 0) n /= x + 1;
        return n == 1;
    }

    private static int[] getPrimes() {
        int[] primes = new int[30];
        int pt = 0;
        for (int i = 2; pt < primes.length; i++) {
            boolean ok = true;
            for (int j = 2; j * j <= i; j++) {
                if (i % j == 0) {
                    ok = false;
                    break;
                }
            }
            if (ok) {
                primes[pt++] = i;
            }
        }
        return primes;
    }

    private static void genProd(int a) {
        int b = a + 1;
        long r = 1;
        while (r < MAXN / b) {
            if (rnd.nextInt(2) == 0) {
                r = r * b;
            } else {
                r = r * a;
            }
        }
        test(r);
    }

    private static void randomTests() {
        for (int i = 0; i < 5; i++) {
            test(Math.abs(rnd.nextLong() % MAXN));
        }
    }

    private static void genPow2() {
        test(2);
        test(1L << 59);
        test(1 << 18);
    }

    private static long genPrime(long near) {
        while (true) {
            long candidate = Math.abs(rnd.nextLong()) % near;
            if (BigInteger.valueOf(candidate).isProbablePrime(100)) {
                return candidate;
            }
        }
    }

    private static void test(long n) {
        try (PrintWriter out = new PrintWriter(String.format("%s/%02d", root, ++nTests))) {
            System.out.println("Writing test " + nTests);
            out.println(n);
        } catch (IOException e) {
            throw new AssertionError(e);
        }
    }

}

